body div td
{
	font-size: 13px;
	font-family: Verdana, Arial, Helvetica, Swiss, Futura, sans-serif;
	color: #000000;
}

#logo {
padding:10px;
width:740px;
background-color: #89EC77;
-moz-border-radius: 8px;
text-align:left;
color:green;
font-size: 22px;
margin: 0px;
}

#content {
 text-align: left;
 margin: 0;
 width: 740px;
}

#pdfblurb {
 font-size: 13px;
 padding:10px;
 width:740px;
 -moz-border-radius: 8px;
}

.tagline {
 font-size: 12px;
 font-style: italic
}

hr {
 border: 0;
 clear: both;
 color: white;
 border-bottom: 1px dotted #A9A9A9;
 margin: 1em;
}

#fax
{
 font-size: 13px;
 margin: 2em;
}

#faxdata
{
  border: 1px;
}

#error.blockquote
{
background-color: #89EC77;
-moz-border-radius: 8px;
}
