   <div id="faxdata">
    <table border="0" align="center">
     <tr>
      <td colspan="2"></td>
      <td rowspan="9" width="20"></td>
      <td rowspan="9" bgcolor="#cccccc" width="1"></td>
      <td rowspan="9" width="20"></td>
      <td colspan="2"></td>
     </tr>
     <tr>
       <td align="right">Received:</td><td>$RECEIVED</td>
       <td align="right">Pages:</td><td>$PAGES</td>
     </tr>
     <tr>
       <td align="right">Sender:</td><td>$SENDER</td>
       <td align="right">Quality:</td><td>$QUALITY</td>
     </tr>
     <tr>
       <td align="right">CID:</td><td>$CALLID1</td>
       <td align="right">Page Size:</td><td>$PAGE</td>
     </tr>
     <tr>
       <td align="right">DID:</td><td>$CALLID2</td>
       <td align="right">TimeToRecv:</td><td>$TIMETORECV</td>
     </tr>
     <tr>
       <td align="right">CommID:</td><td>$COMMID</td>
       <td align="right">SignalRate:</td><td>$SIGNALRATE</td>
     </tr>
     <tr>
       <td align="right">ReceivedOn:</td><td>$DEVICE</td>
       <td align="right">DataFormat:</td><td>$DATAFORMAT</td>
     </tr>
     <tr>
       <td align="right"></td><td></td>
       <td align="right"></td><td></td>
     </tr>
    </table>
   </div>
