<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<html>
<head>
<style type="text/css">
<!--
#INCLUDE css.inc
 -->
</style>
</head>

<body bgcolor="#FFFFFF" align="center">

 <div id="logo">
     <img src="http://people.ifax.com/~aidan/hylafax/templates/images/small_frog.png" style="vertical-align:text-bottom;"> HylaFAX <span class="tagline">The world's most advanced open source fax server</span>
 </div>

 <div id="content" align="center">
