
   <div id="faxdata">
    <table border="0" align="center">
     <tr>
      <td colspan="2"></td>
      <td rowspan="99" width="20"></td>
      <td rowspan="99" bgcolor="#cccccc" width="1"></td>
      <td rowspan="99" width="20"></td>
      <td colspan="2"></td>
     </tr>
     <tr>
       <td align="right">Pages:</td><td>$npages / $totpages</td>
       <td align="right">Number:</td><td>$number</td>
     </tr>
     <tr>
       <td align="right">Page Width:</td><td>$pagewidth (mm)</td>
       <td align="right">Receiver:</td><td>$receiver</td>
     </tr>
     <tr>
       <td align="right">Page Length:</td><td>$pagelength</td>
       <td align="right">Company:</td><td>$company</td>
     </tr>
     <tr>
       <td align="right">Resolution:</td><td>$resolution (lpi)</td>
       <td align="right">Remote Station:</td><td>$csi</td>
     </tr>
     <tr>
       <td align="right">Sender:</td><td>$sender</td>
       <td align="right">Signal Rate:</td><td>$signalrate</td>
     </tr>
     <tr>
       <td align="right">Address</td><td>$mailaddr</td>
       <td align="right">Dials:</td><td>$ndials</td>
     </tr>
     <tr>
       <td align="right">Submitted from:</td><td>$client</td>
       <td align="right">Calls:</td><td>$totdials</td>
     </tr>
     <tr>
       <td align="right">Modem Group:</td><td>$modem</td>
       <td align="right">Dialogs:</td><td>$tottries</td>
     </tr>
     <tr>
       <td align="right">CommID:</td><td>$commid</td>
       <td align="right">Attempts:</td><td>$ntries</td>
     </tr>
     <tr>
       <td align="right"></td><td></td>
       <td align="right"></td><td></td>
     </tr>
    </table>

   </div>
