 <div id="pdfblurb" align="left">
  <blockquote>
    Your fax is attached to this email in PDF format. If you have
    problems decoding this attachment, you may need to install a PDF
    viewer. A common viewer available for many operating systems is
    Adobe's Acrobat reader, which may be
    <a href="http://www.adobe.com/products/acrobat/readstep2.html">
     downloaded here
    </a>.
  </blockquote>
 </div>
